#!/bin/bash

# This script assumes that smartctl is installed on
# the disk in /usr/sbin

log_file=/home/spectracom/log/diskhealth.log

version="$(/bin/cat /etc/motd)"

host_name="$(/bin/hostname)"
log_format="$(/bin/date +"%b %d %T") ${host_name} ${host_name}: [health]"

fail_message="Failed to determine health of disk - Error Code:"
status_message="Disk Maintenance Required:"

ec_day_520=1308960
ec_day_582=110240
ec_rate=${ec_day_582}
ec_threshold=400000000

# Determine Rate of disk wear:
# Only versions between 5.2.0 and 5.8.1 are affected by the special EC Rate
if echo "${version}" | grep -qE "Version (5[.][2-7][.].)|(5[.]8[.][0-1])"; then
    ec_rate=${ec_day_520}
fi

# Create the log file with the correct ownership
touch ${log_file}
chown root:root ${log_file}
chmod 644 ${log_file}

# Gather smart info so we can determine
# if this is a western digital or not
smart_info=$(sudo /usr/sbin/smartctl -i /dev/*da)
ret_code=$?
if [ ${ret_code} -ne 0 ] ; then
    echo "${log_format} ${fail_message} ${ret_code}" >> ${log_file}
    echo "${log_format} ${fail_message} ${ret_code}"
    exit 1
fi

# Check smart info to see if smart attributes are
# available.  If not, print warning immediately
# they are running out-of-warrenty WD card
if echo "${smart_info}" | grep -i "SMART Support is" | grep -i "device lacks SMART capability" ; then
    echo "${log_format} ${fail_message} ${ret_code}" >> ${log_file}
    echo "${log_format} ${fail_message} ${ret_code}"
    exit 4
fi


# Make sure smartctl is on for the CF card
sudo /usr/sbin/smartctl -s on /dev/*da > /dev/null 2>&1
ret_code=$?
if [ ${ret_code} -ne 0 ] ; then
    echo "${log_format} ${fail_message} ${ret_code}" >> ${log_file}
    echo "${log_format} ${fail_message} ${ret_code}"
    exit 2
fi

smart_data=$(sudo /usr/sbin/smartctl -A -v 229,raw64:r543210w,Erase_Count /dev/*da)
ret_code=$?
if [ $? -ne 0 ] ; then
    echo "${log_format} ${fail_message} ${ret_code}" >> ${log_file}
    echo "${log_format} ${fail_message} ${ret_code}"
    exit 3
fi

# Pull erase counts from disk
eraseCounts=$(echo "${smart_data}" | grep -i "Erase_Count" | tr -s " " | cut -d" " -f 10)

# Calculate months remaining
# ec_remaining = 400,000,000 - EC_CURRENT
# months_remaining = ec_remaining / ec-day / 30

ec_remaining=$((${ec_threshold} - ${eraseCounts}))
months_remaining=$((${ec_remaining} / ${ec_rate} / 30))

month_message="${months_remaining} months"
if [ ${months_remaining} -eq 1 ] ; then
    month_message="${months_remaining} month"
elif [ ${months_remaining} -lt 1 ] ; then
    month_message="less than 1 month"
fi

if [ ${months_remaining} -gt 12 ] ; then
    echo "${log_format} Disk Health OK - No maintenance required (${eraseCounts})" >> ${log_file}
    echo "${log_format} Disk Health OK - No maintenance required (${eraseCounts})"
else
    echo "${log_format} ${status_message} ${month_message} (${eraseCounts})" >> ${log_file}
    echo "${log_format} ${status_message} ${month_message} (${eraseCounts})"
fi
